@@ -83,7 +83,7 @@ typedef struct _TrackProperties{
 
 static void 	send_notification			( LXMusicNotification lxn );
 static void 	send_notification_pixbuf		( LXMusicNotification lxn, GdkPixbuf *pixbuf );
-static int 	update_track				( xmmsv_t *value, GtkTreeIter* it );
+static int 	update_track				( xmmsv_t *value, GtkTreeRowReference* ref );
 static int 	on_coll_info_received			( xmmsv_t* value, void* user_data );
 static int 	on_picture_front_received		( xmmsv_t* value, void* user_data );
 static int 	on_playback_started			( xmmsv_t* value, void* user_data );
@@ -202,7 +202,9 @@ static void save_config()
 
 static int on_xmms_quit(xmmsv_t *value, void *user_data)
 {
-    gtk_widget_destroy(main_win);
+    if (main_win)
+        gtk_widget_destroy(main_win);
+    main_win = NULL;
     gtk_main_quit();
     return TRUE;
 }
@@ -224,6 +226,7 @@ void on_quit(GtkAction* act, gpointer user_data)
     else
     {
         gtk_widget_destroy(main_win);
+        main_win = NULL;
         gtk_main_quit();
     }
 }
@@ -275,6 +278,7 @@ void on_about(GtkWidget* mi, gpointer data)
     gtk_about_dialog_set_license( (GtkAboutDialog*)about, "GNU General Public License" );
     gtk_about_dialog_set_website( (GtkAboutDialog*)about, "http://lxde.org/" );
     gtk_window_set_transient_for( (GtkWindow*)about, (GtkWindow*)main_win );
+    gtk_window_set_destroy_with_parent((GtkWindow*)about, TRUE);
     gtk_dialog_run( (GtkDialog*)about );
     gtk_widget_destroy( about );
 }
@@ -509,6 +513,8 @@ void on_preference(GtkAction* act, gpointer data)
         xmmsc_result_notifier_set_full(res, on_pref_dlg_init_widget, g_object_ref(id3v1_encoding), g_object_unref );
         xmmsc_result_unref(res);
 
+        gtk_window_set_transient_for(GTK_WINDOW(dlg), (GtkWindow*)main_win);
+        gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
         if( gtk_dialog_run(GTK_DIALOG(dlg)) == GTK_RESPONSE_OK )
         {
             int i;
@@ -608,9 +614,11 @@ static int on_coll_info_received(xmmsv_t* value, void* user_data)
 	xmmsv_t *id_val;
 	int32_t id;
 	xmmsv_list_get (value, i,  &track_info);
-	xmmsv_dict_get ( track_info, "id", &id_val );
-	xmmsv_get_int ( id_val, &id );
-	g_hash_table_insert( id_to_coll_info, GINT_TO_POINTER( id ), track_info );
+	if (xmmsv_dict_get ( track_info, "id", &id_val ))
+	{
+	    xmmsv_get_int ( id_val, &id );
+	    g_hash_table_insert( id_to_coll_info, GINT_TO_POINTER( id ), track_info );
+	}
     }
 
     gtk_tree_model_get_iter_first( model, &it );
@@ -618,10 +626,14 @@ static int on_coll_info_received(xmmsv_t* value, void* user_data)
     while (gtk_list_store_iter_is_valid( list_store , &it )) 
     {
 	xmmsv_t *track_info;
+        GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(list_store), &it);
+        GtkTreeRowReference *ref = gtk_tree_row_reference_new(GTK_TREE_MODEL(list_store), path);
+        gtk_tree_path_free(path);
 	gtk_tree_model_get( model, &it, COL_ID, &id, -1 );
 	/* find corressponding track info */
 	track_info = (xmmsv_t*) g_hash_table_lookup( id_to_coll_info, GINT_TO_POINTER( id ));
-	update_track( track_info, &it ); 
+	update_track( track_info, ref );
+	gtk_tree_row_reference_free(ref);
 	gtk_tree_model_iter_next( model, &it );
 	i++;
     }
@@ -684,7 +696,11 @@ static int on_track_info_received(xmmsv_t* value, void* user_data)
     if (tp.size){
 	char buf[100];
 	char *size_str;
+#if GLIB_CHECK_VERSION(2, 30, 0)
+        size_str = g_format_size(tp.size);
+#else
 	size_str = g_format_size_for_display( tp.size );
+#endif
 	strcpy( buf, size_str );
 	g_free( size_str );
 
@@ -989,6 +1005,7 @@ void on_add_files( GtkMenuItem* item, gpointer user_data )
                                                   GTK_STOCK_ADD, RESPONSE_ADD, NULL );
     GtkFileFilter* filter;
 
+    gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
     gtk_file_chooser_set_select_multiple( (GtkFileChooser*)dlg, TRUE );
 
     /* add a custom filter which filters autio files */
@@ -1029,6 +1046,7 @@ void on_add_url( GtkMenuItem* item, gpointer user_data )
             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
             GTK_STOCK_OK, GTK_RESPONSE_OK, NULL );
     GtkWidget *entry = gtk_entry_new();
+    gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
     gtk_box_pack_start( (GtkBox*)((GtkDialog*)dlg)->vbox, entry, FALSE, FALSE, 4 );
     gtk_dialog_set_default_response( (GtkDialog*)dlg, GTK_RESPONSE_OK );
     gtk_entry_set_activates_default( (GtkEntry*)entry, TRUE );
@@ -1055,7 +1073,7 @@ void on_add_btn_clicked(GtkButton* btn, gpointer user_data)
 
 static int intcmp( gconstpointer a, gconstpointer b )
 {
-    return (int)b - (int)a;
+    return GPOINTER_TO_INT(b) - GPOINTER_TO_INT(a);
 }
 
 void on_remove_all(GtkAction* act, gpointer user_data)
@@ -1079,7 +1097,7 @@ void on_remove_selected(GtkAction* act, gpointer user_data)
         for( sel = sels; sel; sel = sel->next )
         {
             GtkTreePath* path = (GtkTreePath*)sel->data;
-            sel->data = (gpointer)gtk_tree_path_get_indices( path )[0];
+            sel->data = GINT_TO_POINTER(gtk_tree_path_get_indices( path )[0]);
             gtk_tree_path_free( path );
         }
 
@@ -1092,7 +1110,7 @@ void on_remove_selected(GtkAction* act, gpointer user_data)
         for( sel = sels; sel; sel = sel->next )
         {
             xmmsc_result_t* res;
-            int pos = (int)sel->data;
+            int pos = GPOINTER_TO_INT(sel->data);
             res = xmmsc_playlist_remove_entry( con, cur_playlist, pos );
             xmmsc_result_unref( res );
         }
@@ -1214,31 +1232,40 @@ static void render_num( GtkTreeViewColumn* col, GtkCellRenderer* render,
     g_object_set( render, "text", buf, NULL );
 }
 
-static int update_track( xmmsv_t *value, GtkTreeIter* it )
+static int update_track( xmmsv_t *value, GtkTreeRowReference *ref )
 {
     TrackProperties track_properties;
-    gboolean current_track_updated;
     char time_buf[32];
     int32_t id;
     gchar *guessed_title = NULL;
+    GtkTreePath *path;
+    GtkTreeIter it;
+    gboolean ok;
+
     if( xmmsv_is_error ( value ) ) {
         return FALSE;
     }
 
+    if (!gtk_tree_row_reference_valid(ref)) /* track was removed from the list */
+        return FALSE;
+    path = gtk_tree_row_reference_get_path(ref);
+    ok = gtk_tree_model_get_iter(GTK_TREE_MODEL(list_store), &it, path);
+    gtk_tree_path_free(path);
+    if (!ok)
+        return FALSE;
     if (!get_track_properties( value, &track_properties)) 
 	track_properties.title = guessed_title = guess_title_from_url( track_properties.url );
     timeval_to_str( track_properties.duration/1000, time_buf, G_N_ELEMENTS(time_buf) );
 
-    gtk_list_store_set( list_store, it,
+    gtk_list_store_set( list_store, &it,
                         COL_ARTIST, track_properties.artist,
                         COL_ALBUM, track_properties.album,
                         COL_TITLE, track_properties.title,
                         COL_LEN, time_buf, -1 );
 
-    
-    gtk_tree_model_get( GTK_TREE_MODEL(list_store), it, COL_ID, &id, -1 );
-     current_track_updated = id == cur_track_id;
-     if ( current_track_updated ) 
+
+    gtk_tree_model_get( GTK_TREE_MODEL(list_store), &it, COL_ID, &id, -1 );
+    if (id > 0)
     {
 	LXMusicNotification lxn = lxmusic_do_notify_prepare ( track_properties.artist, track_properties.title,  
 							      _("Now Playing:"), GTK_STATUS_ICON(tray_icon) );
@@ -1277,7 +1304,7 @@ static int on_picture_front_received ( xmmsv_t* value, void* user_data )
     if (picture_front_pixbuf) 
     {
 	send_notification_pixbuf( lxn, picture_front_pixbuf );
-	gdk_pixbuf_unref( picture_front_pixbuf );
+	g_object_unref( picture_front_pixbuf );
     }
     return TRUE;
 }
@@ -1359,8 +1386,11 @@ static gboolean get_track_properties (xmmsv_t *value, TrackProperties *propertie
 static void queue_update_track( uint32_t id, GtkTreeIter* it )
 {
     xmmsc_result_t *res = xmmsc_medialib_get_info( con, id );
-    xmmsc_result_notifier_set_full ( res, (xmmsc_result_notifier_t)update_track, it, (
-					 xmmsc_user_data_free_func_t) gtk_tree_iter_free );
+    GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(list_store), it);
+    GtkTreeRowReference *ref = gtk_tree_row_reference_new(GTK_TREE_MODEL(list_store), path);
+    gtk_tree_path_free(path);
+    xmmsc_result_notifier_set_full ( res, (xmmsc_result_notifier_t)update_track, ref, (
+					 xmmsc_user_data_free_func_t) gtk_tree_row_reference_free );
     xmmsc_result_unref( res );
 }
 
@@ -1659,12 +1689,12 @@ static int on_playlist_content_changed( xmmsv_t* value, void* user_data )
     if( xmmsv_is_error( value ) )
 	return TRUE;
 
-    xmmsv_dict_get( value, "type", &int_value );
-    if( !xmmsv_get_int( int_value, &type ) )
+    if (!xmmsv_dict_get( value, "type", &int_value ) ||
+        !xmmsv_get_int( int_value, &type ))
         return TRUE;
     
-    xmmsv_dict_get( value, "name", &string_value );
-    if ( !xmmsv_get_string( string_value, &name ) )
+    if (!xmmsv_dict_get( value, "name", &string_value ) ||
+        !xmmsv_get_string( string_value, &name ))
 	return TRUE;
 
     if( ! name || !cur_playlist || strcmp(name, cur_playlist) )
@@ -1690,7 +1720,7 @@ static int on_playlist_content_changed( xmmsv_t* value, void* user_data )
 		xmmsv_get_int( int_value, &id );
                 gtk_list_store_insert_with_values( list_store, &it, pos, COL_ID, id, -1 );
                 /* g_debug("playlist_added: %d", id); */
-                queue_update_track( id, gtk_tree_iter_copy( &it ));
+                queue_update_track( id, &it );
             }
             break;
         case XMMS_PLAYLIST_CHANGED_REMOVE:
@@ -1703,10 +1733,14 @@ static int on_playlist_content_changed( xmmsv_t* value, void* user_data )
 		{
                     gtk_list_store_remove( list_store, &it );
 		    /* invalidate currently played track */
-		    xmmsv_dict_get( value, "id", &int_value );
-		    xmmsv_get_int( int_value, &id );
-		    if ( id == cur_track_id ) 
-			cur_track_id = 0;
+		    if (xmmsv_dict_get( value, "id", &int_value ))
+		    {
+		        xmmsv_get_int( int_value, &id );
+		        if ( id == cur_track_id ) 
+			    cur_track_id = 0;
+		    }
+		    else
+		        cur_track_id = 0;
 		}
                 gtk_tree_path_free( path );
 	    }
@@ -1850,7 +1884,7 @@ static void send_notification_pixbuf( LXMusicNotification lxn, GdkPixbuf *pixbuf
 	/* FIXME: Hardcoded notification icon size */
 	GdkPixbuf *scaled_pixbuf = gdk_pixbuf_scale_simple( pixbuf, 64, 64, GDK_INTERP_HYPER );
 	lxmusic_do_notify_pixbuf( lxn, scaled_pixbuf );
-	gdk_pixbuf_unref( scaled_pixbuf );
+	g_object_unref( scaled_pixbuf );
     }
 }
 
@@ -1900,8 +1934,8 @@ static int on_playlist_pos_changed( xmmsv_t* val, void* user_data )
                     gtk_list_store_set(list_store, &it, COL_WEIGHT, PANGO_WEIGHT_BOLD, -1);
 
 		    /* update id of currently played track */
-		    xmmsv_dict_get( val, "id", &int_value );
-		    xmmsv_get_int( int_value, &cur_track_id );
+                    if( xmmsv_dict_get( val, "id", &int_value ) )
+                        xmmsv_get_int( int_value, &cur_track_id );
                 }
 		/* scroll to currently played song */
 		gtk_tree_view_scroll_to_cell(GTK_TREE_VIEW(playlist_view), path, NULL, FALSE, 0.0, 0.0 );
@@ -1959,7 +1993,7 @@ static void get_channel_volumes( const char *key, xmmsv_t *value, void *user_dat
     GSList** volumes = (GSList**)user_data;
     int32_t volume;
     xmmsv_get_int( value , &volume );
-    *volumes = g_slist_prepend(*volumes, (gpointer)volume);
+    *volumes = g_slist_prepend(*volumes, GINT_TO_POINTER(volume));
 }
 
 static void get_channel_volume_names(const char* key, xmmsv_t *value, void *user_data)
@@ -2035,7 +2069,7 @@ static int on_playback_volume_changed( xmmsv_t* value, void* user_data )
 	xmmsv_get_error( value, &error_msg );
 	g_warning( "%s: %s", __func__, error_msg );
     }
-    else if (xmmsv_is_type( value, XMMSV_TYPE_DICT ))
+    else if (value != NULL && xmmsv_is_type( value, XMMSV_TYPE_DICT ))
     {
 	xmmsv_dict_foreach( value, get_channel_volumes, &volumes );
         for( l = volumes; l; l = l->next )
@@ -2058,18 +2092,18 @@ static int on_collection_changed( xmmsv_t* dict, void* user_data )
     /* xmmsc_result_dict_foreach(res, dict_foreach, NULL); */
     
     xmmsv_t *string_value, *int_value;
-    const char *name, *ns;
-    gint32 type;
+    const char *name = NULL, *ns = NULL;
+    gint32 type = -1;
     
     
-    xmmsv_dict_get (dict, "name", &string_value);
-    xmmsv_get_string( string_value, &name);
+    if (xmmsv_dict_get (dict, "name", &string_value))
+        xmmsv_get_string( string_value, &name);
 
-    xmmsv_dict_get (dict, "namespace", &string_value);
-    xmmsv_get_string( string_value, &ns );
+    if (xmmsv_dict_get (dict, "namespace", &string_value))
+        xmmsv_get_string( string_value, &ns );
 
-    xmmsv_dict_get (dict, "type", &int_value);	
-    xmmsv_get_int( int_value, &type );
+    if (xmmsv_dict_get (dict, "type", &int_value))
+        xmmsv_get_int( int_value, &type );
     
     /* g_debug("name=%s, ns=%s, type=%d", name, ns, type); */
 
@@ -2115,7 +2149,7 @@ static int on_media_lib_entry_changed(xmmsv_t* value, void* user_data)
                 if( _id == id )
                 {
                     /* g_debug("found! update: %d", id); */
-                    queue_update_track( id, gtk_tree_iter_copy( &it ) );
+                    queue_update_track( id, &it );
                     break;
                 }
             }while(gtk_tree_model_iter_next(model, &it));
@@ -2326,6 +2360,7 @@ void on_new_playlist(GtkAction* act, gpointer user_data)
             _("Create new playlist"), (GtkWindow*)main_win, GTK_DIALOG_MODAL,
             GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
             GTK_STOCK_OK, GTK_RESPONSE_OK, NULL );
+    gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
     GtkWidget *entry = gtk_entry_new();
     gtk_box_pack_start( (GtkBox*)((GtkDialog*)dlg)->vbox, entry, FALSE, FALSE, 4 );
     gtk_dialog_set_default_response( (GtkDialog*)dlg, GTK_RESPONSE_OK );
